from logging import Handler, LogRecord, StreamHandler
from typing import Any, TextIO

class LogHandle: ...

class PythonVSScriptLoggingBridge(Handler):
    def __init__(self, parent: StreamHandler[TextIO], level: int | str = ...) -> None: ...
    def emit(self, record: LogRecord) -> None: ...

class Error(Exception):
    value: Any
    def __init__(self, value: Any) -> None: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...
